%% -- Piet Mondrian tutorial ----------------------------------------------
% RMT November 2015 
%
% This script is for creating an rgb image using basic matrix manipulation,
% saving it, and using graphical input, segmentation and loops to modify
% it. Each main image representation (rgb, grayscale, binary, indexed) is
% covered.
%
% Some of the figures are commented out; to see the results of each step,
% remove the '%'.
%
%%-------------------------------------------------------------------------

iptsetpref('ImshowBorder', 'loose')
clear
close all

%% 1. Create matrix

I = ones(1000, 1000, 3);
% Creates a 1000x1000 pixel image, with each element equal to 1. The array
% has 1000x1000x3 elements, where the three stacked matrices correspond to
% red, green and blue values ranging between 0 (no colour) and 1 (pure 
% colour). if all rgb values for a pixel equal 1, it is white, and if zero,
% the pixel is black.
% Recall that a semicolon suppresses the output - this is especially
% important for images!

%figure, imshow(I)
%title('I = ones(1000,1000,3) : blank canvas')

% If displaying multiple images, use "figure" to stop overwriting the
% previous image.

%% 2. Specify borders

I(:, 450:470, :) = 0;
%figure, imshow(I)
%title('I(:, 450:470, :) = 0 : vertical line')

I(400:420, :, :) = 0;
I(650:670, :, :) = 0;
% Recall that pressing up on the keypad when using the command window will
% repeat the previous entries; you can edit these to make typing faster.
% Unlike terminal commands, you can click and edit instead of using left
% and right keys.

I(670:length(I(:,:,1)), 100:115, :) = 0;
%figure, imshow(I)
%title('I(670:length(I(:,:,1)), 100:115, :) = 0')

I(670:length(I(:,:,1)), 775:790, :) = 0;
%figure, imshow(I)
%title('I(670:length(I(:,:,1)), 775:790, :) = 0')

I(935:950, 470:775, :) = 0;
%figure, imshow(I)
%title('I(935:950, 470:775, :) = 0')

%% 3. Colour in manually

I(1:400, 1:450, 2:3) = 0;
% This leaves just the r pane equal to one, resulting in a red square.
%figure, imshow(I)
%title('I(1:400, 450:470, 2:3) = 0 : red square')

I(670:935, 470:775, 1:2) = 0;
%figure, imshow(I)
%title('I(670:935, 470:775, 1:2) = 0 : blue square')

I(670:length(I(:,:,1)), 1:100, 3) = 0;
%figure, imshow(I)
%title('I(670:length(I(:,:,1)), 1:100, 3) = 0 : yellow square')
% By having the rgb of each pixel in this square as pure red and green, we
% get yellow.

%% 4. Saving and loading data

save_ans = input('Save image? y/n [y] ', 's');
% This is a simple method for entering commands during a script instead of
% changing parameters and resaving, or running more scripts.
if (isempty(save_ans) || save_ans == 'y')
    figure, imshow(I)
    title('Composition in C')
    saveas(gcf, 'Composition-C.fig');
    % gcf corresponds to the "current" figure. Here we are saving the
    % figure as a Matlab figure.
    imwrite(I, 'Composition-C.tif')
    % This is a much nicer way of saving images, using the Image Processing
    % Toolbox.
    % To save the Matlab array, use save('Composition-C.mat', 'I');
end
clear save_ans

% A note on saving and loading data:
% If you want to load an image and continue working on it, note that it is
% a good idea to save the raw data or use the image processing 
% toolbox commands imwrite, imsave and imread rather than as a Matlab 
% figure (.fig) above.
% For example, double clicking on the file 'Composition-C.fig' and using
% the commands I2 = frame2im(getframe(gcf)) will result in data loss
% compared to the original array (check using the size() function, or
% check the Workspace window).

% Also, watch out in the Workspace window the value type for each variable,
% which might be changed when importing data or using certain functions.
% Matlab will get angry if you try to add a "double" matrix with a "uint8" 
% one. It's very easy to change these: just use
% matrixname = im2uint8(matrixname) etc.

%% 5. Creating a colouring-in script with graphical input

close all

%Create a new image with just the border from before
Ig = rgb2gray(I);
% rgb2gray() converts the nxmx3 rgb image to a nxm grayscale one.
P = ones(size(Ig));
P(Ig == 0) = 0;
P(:,:,2) = P(:,:,1);
P(:,:,3) = P(:,:,1);
clear Ig

exit = 0;
while exit ~= 1

figure, imshow(P)
title('Select area and specify colour')
display('Position the cursor on a square to be coloured, and press r, g or')
display('b to flip the value of the corresponding colour between 0 and 1.')
display('To finish, left click on the image.')
[x, y, button] = ginput(1);
close all
% The variable "button" is saved in ASCII format. So the letters r, g and b
% are saved as 114, 103 and 98 respectively. A left click is saved as 1.
if button == 1
    exit_ans = input('Are you sure? y/n [n] ', 's');
    if exit_ans == 'y'
        figure, imshow(P)
        exit = 1;
    end
else
    % Next, we want to find the pixels corresponding to this selected 
    % region. To do so, we'll create a binary image from the gray one 
    % according to the selected value, index it through segmentation and 
    % then change the values of this region accordingly.
    
    % There are two important points to bear in mind when using graphical
    % inputs. Firstly, the coordinates are spatial and not intrinsic 
    % (pixel) coordinates, so we have to round them when referring to them
    % in a matrix. We also have to flip x and y when doing so.
    n = round(x, 0);
    m = round(y, 0);
    Pg = rgb2gray(P);
    thresh_val = Pg(m, n);
    % (m,n instead of y,x is just for debugging, ignore)
    
    % Grayscale unsigned 8-bit integer (uint8) images have values ranging
    % from 0 (black) to 255 (white). For type 'double', the values range
    % from 0 to 1.
    % To convert between the two correctly, use im2uint8() etc.
    
    % Now we isolate the parts of the image which have the same value as
    % the point selected.
    Pt = ones(size(Pg));
    Pt(Pg ~= thresh_val) = 0;
    %figure, imshow(Pt)
    %title('Matrix of selected values')
    
    % There might be more than one area with that value, so we use a simple
    % segmentation method to create a label matrix for each area. See the
    % Matlab documentation for more info on bwconncomp().
    Pt = im2bw(Pt);
    CC = bwconncomp(Pt);
    L = labelmatrix(CC);
    %figure, imagesc(L)
    %title('Label matrix for selected value')
    % Use imagesc() instead of imshow() to produce a scaled image. Using
    % the latter function will give a dark image as the label matrix is 
    % type uint8 with low value integers as labels.
    
    % Now, we find the labelled area that was selected using ginput().
    Pt_selectedarea = zeros(size(Pt));
    Pt_selectedarea(L == L(m, n)) = 1;
    %figure, imshow(Pt_selectedarea)
    %title('Selected area')
    
    Pt_selectedarea = im2bw(Pt_selectedarea);
    if button == 114 %(red)
        P(:,:,1) = abs(P(:,:,1) - Pt_selectedarea);
    elseif button == 103 %(green)
        P(:,:,2) = abs(P(:,:,2) - Pt_selectedarea);
    elseif button == 98 %(blue)
        P(:,:,3) = abs(P(:,:,3) - Pt_selectedarea);
    end
    
    % Debugging exercise: how would we stop the border from being coloured
    % after all r,g,b is removed from a square?
    
end
end
clear exit exit_ans
clear m n x y button thresh_val